<?php

/**
 * Graph plugin
 *
 *
 * Created: 2007-02-18
 * Last update: 2007-05-20
 *
 * @link http://deboutv.free.fr/mantis/
 * @author DEBOUT Vincent  <deboutv@free.fr>
 */

require( dirname( dirname( dirname( __FILE__ ) ) ) . DIRECTORY_SEPARATOR . 'core.php' );

$t_core_path = config_get( 'core_path' );

require_once( $t_core_path . 'graph_api.php' );

access_ensure_project_level( config_get( 'view_summary_threshold' ) );

function plugins_graph_convert_key( $p_value, $p_field, $p_project_id = 0 ) {
    global $t_project_id;
    global $t_choices;

    $t_result = '';
    if ( $p_field == 'reporter_id' || $p_field == 'handler_id' ) {
        $t_result = user_get_name( $p_value );
    } else if ( $p_field == 'project_id' ) {
        $t_result = project_get_name( $p_value );
    } else if ( ( $p_field == 'target_version' || $p_field == 'version' || $p_field == 'fixed_in_version' || $p_field == 'category' ) && $t_project_id == 0 ) {
        $t_result = project_get_name( $p_project_id ) . ' - ' . $p_value;
    } else if ( in_array( $p_field, array( 'status', 'priority', 'severity', 'eta', 'reproducibility', 'resolution' ) ) ) {
        $t_result = get_enum_element( $p_field, $p_value );
    } else if ( in_array( $p_field, $t_choices ) ) {
        $t_result = $p_value;
    } else if ( ereg( 'custom_', $p_field ) ) {
        $t_id = (int)str_replace( 'custom_', '', $p_field );
        $t_value = $p_value;
        if ( custom_field_type( $t_id ) == CUSTOM_FIELD_TYPE_DATE ) {
            $t_value = date( config_get( 'short_date_format' ), (int)$t_value );
        }
        if ( $p_project_id != 0 ) {
            $t_result = project_get_name( $p_project_id ) . ' - ' . $t_value;
        } else {
            $t_result = $t_value;
        }
    }
    return $t_result;
}

$t_width = gpc_get_int( 'width', 300 );
$t_id = gpc_get_int( 'id' );
$t_ar = config_get( 'graph_bar_aspect' );

$t_row = plugins_graph_get_row( $t_id );

$t_project_id = helper_get_current_project();

$t_select = 'id';
$t_table = '';
$t_where = '';
$t_group = '';

if ( $t_row['yaxis'] == 'bug_count' ) {
    $t_select = 'COUNT(b.id)';
    $t_field = $t_select;
    $t_table = config_get( 'mantis_bug_table' ) . ' b';
} else if ( ereg( 'custom_', $t_row['yaxis'] ) ) {
    $t_select = 'COUNT(b.id), SUM(c.value)';
    $t_field = 'SUM(c.value)';
    $t_table = config_get( 'mantis_bug_table' ) . ' b LEFT JOIN ' . config_get( 'mantis_custom_field_string_table' ) . ' c ON b.id=c.bug_id';
}
if ( $t_project_id != 0 && $t_row['xaxis'] != 'project_id' ) {
    $t_where = ' WHERE b.project_id=' . $t_project_id;
}
if ( ereg( 'custom_', $t_row['yaxis'] ) ) {
    if ( $t_where != '' ) {
        $t_where .= ' AND c.field_id=' . str_replace( 'custom_', '', $t_row['yaxis'] );
    } else {
        $t_where = ' WHERE c.field_id=' . str_replace( 'custom_', '', $t_row['yaxis'] );
    }
}
$t_choices = array( 'reporter_id', 'handler_id', 'category', 'project_id', 'status', 'severity', 'priority', 'reproducibility', 'resolution', 'projection', 'eta', 'os', 'version', 'fixed_in_version' );
if ( $g_mantis_is_11x_release ) {
    $t_choices[] = 'target_version';
}
if ( $t_row['xaxis'] == 'category' || $t_row['xaxis'] == 'target_version' || $t_row['xaxis'] == 'version' || $t_row['xaxis'] == 'fixed_in_version' ) {
    $t_group = ' GROUP BY b.' . $t_row['xaxis'] . ', b.project_id';
    if ( $t_select != '' ) {
        $t_select .= ', ';
    }
    $t_select .= $t_row['xaxis'] . ', project_id';
} else if ( in_array( $t_row['xaxis'], $t_choices ) ) {
    $t_group = ' GROUP BY b.' . $t_row['xaxis'];
    if ( $t_select != '' ) {
        $t_select .= ', ';
    }
    $t_select .= $t_row['xaxis'];
} else if ( ereg( 'custom_', $t_row['xaxis'] ) ) {
    $t_group = ' GROUP BY c.value, b.project_id';
    if ( $t_select != '' ) {
        $t_select .= ', ';
    }
    $t_select .= 'c.value, b.project_id';
    if ( $t_where != '' ) {
        $t_where .= ' AND ';
    } else {
        $t_where = ' WHERE ';
    }
    $t_where .= 'c.field_id=' . str_replace( 'custom_', '', $t_row['xaxis'] );
    $t_table .= ' LEFT JOIN ' . config_get( 'mantis_custom_field_string_table' ) . ' c ON b.id=c.bug_id';
}

$t_query = 'SELECT ' . $t_select . ' FROM ' . $t_table . $t_where . $t_group;

$t_metrics = array();
$t_temp_y = array();
$t_temp_x = array();
if ( !ereg( 'custom_', $t_row['yaxis'] ) || !ereg( 'custom_', $t_row['xaxis'] ) ) {
    if ( db_is_connected() ) {
        $t_result = db_query( $t_query );
        while( $t_rw = db_fetch_array( $t_result ) ) {
            if ( ereg( 'custom_', $t_row['xaxis'] ) ) {
                $t_key = 'value';
            } else {
                $t_key = $t_row['xaxis'];
            }
            if ( isset( $t_rw['project_id'] ) && $t_project_id == 0 ) {
                $t_metrics[plugins_graph_convert_key( $t_rw[$t_key], $t_row['xaxis'], $t_rw['project_id'] )] = $t_rw[$t_field];
            } else {
                $t_metrics[plugins_graph_convert_key( $t_rw[$t_key], $t_row['xaxis'] )] = $t_rw[$t_field];
            }
            if ( ereg( 'custom_', $t_row['yaxis'] ) ) {
                $t_temp_y[$t_rw[$t_row['xaxis']]] = $t_rw['COUNT(b.id)'];
            }
            if ( ereg( 'custom_', $t_row['xaxis'] ) ) {
                $t_temp_x[$t_key] = $t_rw['COUNT(b.id)'];
            }
        }
    }

    if ( ereg( 'custom_', $t_row['yaxis'] ) ) {
        $t_custom_id = (int)str_replace( 'custom_', '', $t_row['yaxis'] );
        if ( (float)custom_field_get_field( $t_custom_id, 'default_value' ) != 0.0 ) {
            if ( custom_field_get_field( $t_custom_id, 'type' ) == CUSTOM_FIELD_TYPE_NUMERIC ) {
                $t_default = (int)custom_field_get_field( $t_custom_id, 'default_value' );
            } else {
                $t_default = (float)custom_field_get_field( $t_custom_id, 'default_value' );
            }
            if ( $t_project_id == 0 ) {
                $t_project_ids = custom_field_get_project_ids( $t_custom_id );
            } else {
                $t_project_ids = array( $t_project_id );
            }
            $t_query = 'SELECT COUNT(b.id), ' . $t_row['xaxis'] . ', b.project_id FROM ' . config_get( 'mantis_bug_table' ) . ' b WHERE b.project_id=' . $t_project_ids[0];
            for( $i=1; $i<count( $t_project_ids ); $i++ ) {
                $t_query .= ' OR b.project_id=' . $t_project_ids[$i];
            }
            $t_query .= $t_group;
            $t_result = db_query( $t_query );
            while( $t_rw = db_fetch_array( $t_result ) ) {
                if ( isset( $t_rw['project_id'] ) ) {
                    $t_field = plugins_graph_convert_key( $t_rw[$t_row['xaxis']], $t_row['xaxis'], $t_rw['project_id'] );
                } else {
                    $t_field = plugins_graph_convert_key( $t_rw[$t_row['xaxis']], $t_row['xaxis'] );
                }
                if ( isset( $t_metrics[$t_field] ) ) {
                    $t_metrics[$t_field] += $t_default * ( $t_rw['COUNT(b.id)'] - $t_temp_y[$t_rw[$t_row['xaxis']]] );
                } else {
                    $t_metrics[$t_field] = $t_default * $t_rw['COUNT(b.id)'];
                }
            }
        }
    }

    if ( ereg( 'custom_', $t_row['xaxis'] ) ) {
        $t_custom_id = (int)str_replace( 'custom_', '', $t_row['xaxis'] );
        if ( custom_field_get_field( $t_custom_id, 'default_value' ) != '' ) {
            $t_default = custom_field_get_field( $t_custom_id, 'default_value' );
            if ( $t_project_id == 0 ) {
                $t_project_ids = custom_field_get_project_ids( $t_custom_id );
            } else {
                $t_project_ids = array( $t_project_id );
            }
            $t_query = 'SELECT COUNT(b.id), b.project_id FROM ' . config_get( 'mantis_bug_table' ) . ' b WHERE b.project_id=' . $t_project_ids[0];
            for( $i=1; $i<count( $t_project_ids ); $i++ ) {
                $t_query .= ' OR b.project_id=' . $t_project_ids[$i];
            }
            $t_query .= ' GROUP BY b.project_id';
            $t_result = db_query( $t_query );
            while( $t_rw = db_fetch_array( $t_result ) ) {
                if ( isset( $t_rw['project_id'] ) && $t_project_id == 0 ) {
                    $t_field = plugins_graph_convert_key( $t_default, $t_row['xaxis'], $t_rw['project_id'] );
                } else {
                    $t_field = plugins_graph_convert_key( $t_default, $t_row['xaxis'] );
                }
                if ( isset( $t_metrics[$t_field] ) ) {
                    $t_metrics[$t_field] += ( $t_rw['COUNT(b.id)'] - $t_temp_x[$t_field] );
                } else {
                    $t_metrics[$t_field] = $t_rw['COUNT(b.id)'];
                }
            }
        }
    }
} else {
    $t_custom_id_y = (int)str_replace( 'custom_', '', $t_row['yaxis'] );
    $t_custom_id_x = (int)str_replace( 'custom_', '', $t_row['xaxis'] );
    $t_project_ids = array_uintersect( custom_field_get_project_ids( $t_custom_id_x ), custom_field_get_project_ids( $t_custom_id_y ), 'strcasecmp' );
    if ( $t_project_id != 0 && in_array( $t_project_id, $t_project_ids ) ) {
        $t_project_ids = array( $t_project_id );
    } else if ( $t_project_id != 0 && !in_array( $t_project_id, $t_project_ids ) ) {
        $t_project_ids = array();
    }
    if ( count( $t_project_ids ) > 0 ) {
        $t_query = 'SELECT b.id, c.field_id, c.value, b.project_id FROM demo_bug_table b LEFT JOIN demo_custom_field_string_table c ON b.id=c.bug_id WHERE ( b.project_id=' . $t_project_ids[0];
        for( $i=1; $i<count( $t_project_ids ); $i++ ) {
            $t_query .= ' OR b.project_id=' . $t_project_ids[$i];
        }
        $t_query .= ' ) AND ( c.field_id=' . $t_custom_id_x . ' OR c.field_id=' . $t_custom_id_y . ' ) ORDER BY b.id ASC, c.field_id';
        if ( max( $t_custom_id_x, $t_custom_id_y ) == $t_custom_id_x ) {
            $t_query .= ' DESC';
        } else {
            $t_query .= ' ASC';
        }
        $t_result = db_query( $t_query );
        $t_default_x = custom_field_get_field( $t_custom_id_x, 'default_value' );
        if ( custom_field_get_field( $t_custom_id_y, 'type' ) == CUSTOM_FIELD_TYPE_NUMERIC ) {
            $t_default_y = (int)custom_field_get_field( $t_custom_id_y, 'default_value' );
        } else {
            $t_default_y = (float)custom_field_get_field( $t_custom_id_y, 'default_value' );
        }
        $t_last_id = 0;
        $t_last_project_id = 0;
        $t_key = '';
        $t_no_default = false;
        $t_count = array();
        while( $t_rw = db_fetch_array( $t_result ) ) {
            if ( !isset( $t_count[$t_rw['project_id']] ) ) {
                $t_count[$t_rw['project_id']] = 0;
            }
            if ( isset( $t_rw['project_id'] ) && $t_project_id == 0 ) {
                $t_ky = plugins_graph_convert_key( $t_key, $t_row['xaxis'], $t_rw['project_id'] );
            } else {
                $t_ky = plugins_graph_convert_key( $t_key, $t_row['xaxis'] );
            }
            if ( isset( $t_rw['project_id'] ) && $t_project_id == 0 ) {
                $t_default_xk = plugins_graph_convert_key( $t_default_x, $t_row['xaxis'], $t_rw['project_id'] );
            } else {
                $t_default_xk = plugins_graph_convert_key( $t_default_x, $t_row['xaxis'] );
            }
            if ( $t_last_id == $t_rw['id'] ) {
                if ( isset( $t_metrics[$t_ky] ) ) {
                    $t_metrics[$t_ky] += $t_rw['value'];
                } else {
                    $t_metrics[$t_ky] = $t_rw['value'];
                }
                $t_count[$t_rw['project_id']]++;
                $t_no_default = true;
            } else {
                $t_next_default = false;
                if ( $t_last_id == 0 ) {
                    if ( $t_rw['field_id'] == $t_custom_id_y ) {
                        $t_metrics[$t_default_xk] = $t_rw['value'];
                        $t_next_default = true;
                        $t_count[$t_rw['project_id']]++;
                    } else {
                        $t_key = $t_rw['value'];
                    }
                } else {
                    if ( $t_no_default ) {
                       if ( $t_rw['field_id'] == $t_custom_id_y ) {
                           if ( isset( $t_metrics[$t_default_xk] ) ) {
                               $t_metrics[$t_default_xk] += $t_rw['value'];
                           } else {
                               $t_metrics[$t_default_xk] = $t_rw['value'];
                           }
                           $t_count[$t_rw['project_id']]++;
                           $t_next_default = true;
                       } else {
                           $t_key = $t_rw['value'];
                       }
                   } else {
                       if ( isset( $t_metrics[$t_ky] ) ) {
                           $t_metrics[$t_ky] += $t_default_y;
                       } else {
                           $t_metrics[$t_ky] = $t_default_y;
                       }
                       $t_count[$t_last_project_id]++;
                       if ( $t_rw['field_id'] == $t_custom_id_y ) {
                           if ( isset( $t_metrics[$t_default_xk] ) ) {
                               $t_metrics[$t_default_xk] += $t_rw['value'];
                           } else {
                               $t_metrics[$t_default_xk] = $t_rw['value'];
                           }
                           $t_count[$t_rw['project_id']]++;
                           $t_next_default = true;
                       } else {
                           $t_key = $t_rw['value'];
                       }
                   }
                }
                $t_no_default = $t_next_default;
            }
            $t_last_id = $t_rw['id'];
            $t_last_project_id = $t_rw['project_id'];
        }
        if ( !$t_no_default ) {
            if ( isset( $t_rw['project_id'] ) && $t_project_id == 0 ) {
                $t_ky = plugins_graph_convert_key( $t_key, $t_row['xaxis'], $t_rw['project_id'] );
            } else {
                $t_ky = plugins_graph_convert_key( $t_key, $t_row['xaxis'] );
            }
            $t_count[$t_last_project_id]++;
            if ( isset( $t_metrics[$t_ky] ) ) {
                $t_metrics[$t_ky] += $t_default_y;
            } else {
                $t_metrics[$t_ky] = $t_default_y;
            }
        }
        $t_query = 'SELECT COUNT(b.id), b.project_id FROM demo_bug_table b WHERE ( b.project_id=' . $t_project_ids[0];
        for( $i=1; $i<count( $t_project_ids ); $i++ ) {
            $t_query .= ' OR b.project_id=' . $t_project_ids[$i];
        }
        $t_query .= ' ) GROUP BY b.project_id';
        $t_result = db_query( $t_query );
        while( $t_rw = db_fetch_array( $t_result ) ) {
            if ( isset( $t_rw['project_id'] ) && $t_project_id == 0 ) {
                $t_default_xk = plugins_graph_convert_key( $t_default_x, $t_row['xaxis'], $t_rw['project_id'] );
            } else {
                $t_default_xk = plugins_graph_convert_key( $t_default_x, $t_row['xaxis'] );
            }
            if ( isset( $t_metrics[$t_default_xk] ) ) {
                $t_metrics[$t_default_xk] += ( $t_rw['COUNT(b.id)'] - $t_count[$t_rw['project_id']] ) * $t_default_y;
            } else {
                $t_metrics[$t_default_xk] = ( $t_rw['COUNT(b.id)'] - $t_count[$t_rw['project_id']] ) * $t_default_y;
            }
        }
    }
}

if ( $t_row['type'] == 0 ) {
    graph_bar( $t_metrics, utf8_encode( $t_row['title'] ), $t_width, $t_width * $t_ar );
} else {
    graph_pie( $t_metrics, utf8_encode( $t_row['title'] ), $t_width );
}

?>